using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Web;
using System.Web.Profile;

namespace XmlProviders
{
    /// <summary>
    /// 
    /// </summary>
    public class XmlProfileProvider : System.Web.Profile.ProfileProvider
    {
        private string _dataStorePath;
        private string _applicationName;

        /// <summary>
        /// Gets or sets the data store path.
        /// </summary>
        /// <value>The data store path.</value>
        public string DataStorePath
        {
            get { return _dataStorePath; }
            set { _dataStorePath = value; }
        }

        private string DataStoreFile
        {
            get
            {
                string path;
                if (HttpContext.Current != null)
                {
                    path = HttpContext.Current.Server.MapPath(DataStorePath);
                }
                else
                {
                    path = System.Web.Hosting.HostingEnvironment.ApplicationPhysicalPath;
                    string dataStorePath = DataStorePath.Replace("~/", string.Empty);
                    path = System.IO.Path.Combine(path, dataStorePath);

                }

                return System.IO.Path.Combine(path, string.Format("{0}_Profiles.xml", _applicationName));
            }
        }

        /// <summary>
        /// Initializes the provider.
        /// </summary>
        /// <param name="name">The friendly name of the provider.</param>
        /// <param name="config">A collection of the name/value pairs representing the provider-specific attributes specified in the configuration for this provider.</param>
        /// <exception cref="T:System.ArgumentNullException">The name of the provider is null.</exception>
        /// <exception cref="T:System.InvalidOperationException">An attempt is made to call <see cref="M:System.Configuration.Provider.ProviderBase.Initialize(System.String,System.Collections.Specialized.NameValueCollection)"></see> on a provider after the provider has already been initialized.</exception>
        /// <exception cref="T:System.ArgumentException">The name of the provider has a length of zero.</exception>
        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            if (config == null) { throw new ArgumentNullException("config"); }
            if (String.IsNullOrEmpty(name)) { name = "XmlProfileProvider"; }

            base.Initialize(name, config);

            if (!String.IsNullOrEmpty(config["dataStorePath"])) { this._dataStorePath = config["dataStorePath"]; }
            if (!String.IsNullOrEmpty(config["applicationName"])) { this._applicationName = config["applicationName"]; }
        }

        /// <summary>
        /// When overridden in a derived class, deletes all user-profile data for profiles in which the last activity date occurred before the specified date.
        /// </summary>
        /// <param name="authenticationOption">One of the <see cref="T:System.Web.Profile.ProfileAuthenticationOption"></see> values, specifying whether anonymous, authenticated, or both types of profiles are deleted.</param>
        /// <param name="userInactiveSinceDate">A <see cref="T:System.DateTime"></see> that identifies which user profiles are considered inactive. If the <see cref="P:System.Web.Profile.ProfileInfo.LastActivityDate"></see>  value of a user profile occurs on or before this date and time, the profile is considered inactive.</param>
        /// <returns>
        /// The number of profiles deleted from the data source.
        /// </returns>
        public override int DeleteInactiveProfiles(System.Web.Profile.ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate)
        {
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);
            List<Data.Profile> toBeRemoved = new List<Data.Profile>();

            foreach (Data.Profile p in profiles)
            {
                switch (authenticationOption)
                {
                    case ProfileAuthenticationOption.All:
                        if (p.LastActivityDate <= userInactiveSinceDate) { toBeRemoved.Add(p); }
                        break;
                    case ProfileAuthenticationOption.Anonymous:
                        if (p.LastActivityDate <= userInactiveSinceDate && p.IsAnonymous) { toBeRemoved.Add(p); }
                        break;
                    case ProfileAuthenticationOption.Authenticated:
                        if (p.LastActivityDate <= userInactiveSinceDate && !p.IsAnonymous) { toBeRemoved.Add(p); }
                        break;
                }
            }

            for (int i = 0; i < toBeRemoved.Count; i++)
            {
                profiles.Remove(toBeRemoved[i]);
            }

            return toBeRemoved.Count;
        }

        /// <summary>
        /// When overridden in a derived class, deletes profile properties and information for profiles that match the supplied list of user names.
        /// </summary>
        /// <param name="usernames">A string array of user names for profiles to be deleted.</param>
        /// <returns>
        /// The number of profiles deleted from the data source.
        /// </returns>
        public override int DeleteProfiles(string[] usernames)
        {
            int counter = 0;
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);

            foreach (string name in usernames)
            {
                Collection<Data.Profile> p = profiles.FindByUserName(name, ProfileAuthenticationOption.All);
                if (p.Count > 0)
                {
                    for (int i = 0; i < p.Count; i++)
                    {
                        profiles.Remove(p[i]);
                        counter++;
                    }
                }
            }

            profiles.Save(this.DataStoreFile);

            return counter;
        }

        /// <summary>
        /// When overridden in a derived class, deletes profile properties and information for the supplied list of profiles.
        /// </summary>
        /// <param name="profiles">A <see cref="T:System.Web.Profile.ProfileInfoCollection"></see>  of information about profiles that are to be deleted.</param>
        /// <returns>
        /// The number of profiles deleted from the data source.
        /// </returns>
        public override int DeleteProfiles(System.Web.Profile.ProfileInfoCollection profiles)
        {
            List<string> usernames = new List<string>();
            foreach (System.Web.Profile.ProfileInfo pi in profiles)
            {
                usernames.Add(pi.UserName);
            }

            return DeleteProfiles(usernames.ToArray());
        }

        /// <summary>
        /// When overridden in a derived class, retrieves profile information for profiles in which the last activity date occurred on or before the specified date and the user name matches the specified user name.
        /// </summary>
        /// <param name="authenticationOption">One of the <see cref="T:System.Web.Profile.ProfileAuthenticationOption"></see> values, specifying whether anonymous, authenticated, or both types of profiles are returned.</param>
        /// <param name="usernameToMatch">The user name to search for.</param>
        /// <param name="userInactiveSinceDate">A <see cref="T:System.DateTime"></see> that identifies which user profiles are considered inactive. If the <see cref="P:System.Web.Profile.ProfileInfo.LastActivityDate"></see> value of a user profile occurs on or before this date and time, the profile is considered inactive.</param>
        /// <param name="pageIndex">The index of the page of results to return.</param>
        /// <param name="pageSize">The size of the page of results to return.</param>
        /// <param name="totalRecords">When this method returns, contains the total number of profiles.</param>
        /// <returns>
        /// A <see cref="T:System.Web.Profile.ProfileInfoCollection"></see> containing user profile information for inactive profiles where the user name matches the supplied usernameToMatch parameter.
        /// </returns>
        public override System.Web.Profile.ProfileInfoCollection FindInactiveProfilesByUserName(System.Web.Profile.ProfileAuthenticationOption authenticationOption, string usernameToMatch, DateTime userInactiveSinceDate, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// When overridden in a derived class, retrieves profile information for profiles in which the user name matches the specified user names.
        /// </summary>
        /// <param name="authenticationOption">One of the <see cref="T:System.Web.Profile.ProfileAuthenticationOption"></see> values, specifying whether anonymous, authenticated, or both types of profiles are returned.</param>
        /// <param name="usernameToMatch">The user name to search for.</param>
        /// <param name="pageIndex">The index of the page of results to return.</param>
        /// <param name="pageSize">The size of the page of results to return.</param>
        /// <param name="totalRecords">When this method returns, contains the total number of profiles.</param>
        /// <returns>
        /// A <see cref="T:System.Web.Profile.ProfileInfoCollection"></see> containing user-profile information for profiles where the user name matches the supplied usernameToMatch parameter.
        /// </returns>
        public override System.Web.Profile.ProfileInfoCollection FindProfilesByUserName(System.Web.Profile.ProfileAuthenticationOption authenticationOption, string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);
            ProfileInfoCollection pic = profiles.CreateProfileInfoCollection(authenticationOption);
            List<ProfileInfo> lst = new List<ProfileInfo>();
            foreach (ProfileInfo p in pic)
            {
                if (p.UserName.Contains(usernameToMatch)) { lst.Add(p); }
            }

            totalRecords = lst.Count;

            ProfileInfoCollection output = new ProfileInfoCollection();
            for (int i = (pageIndex * pageSize); i < ((pageIndex * pageSize) + pageSize); i++)
            {
                if (lst.Count < i) { break; }
                output.Add(lst[i]);
            }

            return output;
        }

        /// <summary>
        /// When overridden in a derived class, retrieves user-profile data from the data source for profiles in which the last activity date occurred on or before the specified date.
        /// </summary>
        /// <param name="authenticationOption">One of the <see cref="T:System.Web.Profile.ProfileAuthenticationOption"></see> values, specifying whether anonymous, authenticated, or both types of profiles are returned.</param>
        /// <param name="userInactiveSinceDate">A <see cref="T:System.DateTime"></see> that identifies which user profiles are considered inactive. If the <see cref="P:System.Web.Profile.ProfileInfo.LastActivityDate"></see>  of a user profile occurs on or before this date and time, the profile is considered inactive.</param>
        /// <param name="pageIndex">The index of the page of results to return.</param>
        /// <param name="pageSize">The size of the page of results to return.</param>
        /// <param name="totalRecords">When this method returns, contains the total number of profiles.</param>
        /// <returns>
        /// A <see cref="T:System.Web.Profile.ProfileInfoCollection"></see> containing user-profile information about the inactive profiles.
        /// </returns>
        public override System.Web.Profile.ProfileInfoCollection GetAllInactiveProfiles(System.Web.Profile.ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate, int pageIndex, int pageSize, out int totalRecords)
        {
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);

            ProfileInfoCollection pic = profiles.CreateProfileInfoCollection(authenticationOption);
            List<ProfileInfo> lst = new List<ProfileInfo>();

            foreach (ProfileInfo pi in pic)
            {
                if (pi.LastActivityDate <= userInactiveSinceDate)
                {
                    lst.Add(pi);
                }
            }

            totalRecords = lst.Count;
            int counter = 0;

            ProfileInfoCollection newPic = new ProfileInfoCollection();
            foreach (ProfileInfo pi in lst)
            {
                if (counter > (pageIndex * pageSize) && counter < ((pageIndex * pageSize) + pageSize))
                {
                    newPic.Add(pi);
                }
                counter++;
            }

            return newPic;
        }

        /// <summary>
        /// When overridden in a derived class, retrieves user profile data for all profiles in the data source.
        /// </summary>
        /// <param name="authenticationOption">One of the <see cref="T:System.Web.Profile.ProfileAuthenticationOption"></see> values, specifying whether anonymous, authenticated, or both types of profiles are returned.</param>
        /// <param name="pageIndex">The index of the page of results to return.</param>
        /// <param name="pageSize">The size of the page of results to return.</param>
        /// <param name="totalRecords">When this method returns, contains the total number of profiles.</param>
        /// <returns>
        /// A <see cref="T:System.Web.Profile.ProfileInfoCollection"></see> containing user-profile information for all profiles in the data source.
        /// </returns>
        public override System.Web.Profile.ProfileInfoCollection GetAllProfiles(System.Web.Profile.ProfileAuthenticationOption authenticationOption, int pageIndex, int pageSize, out int totalRecords)
        {
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);

            ProfileInfoCollection pic = profiles.CreateProfileInfoCollection(authenticationOption);
            ProfileInfoCollection newPic = new ProfileInfoCollection();

            totalRecords = pic.Count;
            int counter = 0;

            foreach (ProfileInfo pi in pic)
            {
                if (counter > (pageIndex * pageSize) && counter < ((pageIndex * pageSize) + pageSize))
                {
                    newPic.Add(pi);
                }
                counter++;
            }

            return newPic;
        }

        /// <summary>
        /// When overridden in a derived class, returns the number of profiles in which the last activity date occurred on or before the specified date.
        /// </summary>
        /// <param name="authenticationOption">One of the <see cref="T:System.Web.Profile.ProfileAuthenticationOption"></see> values, specifying whether anonymous, authenticated, or both types of profiles are returned.</param>
        /// <param name="userInactiveSinceDate">A <see cref="T:System.DateTime"></see> that identifies which user profiles are considered inactive. If the <see cref="P:System.Web.Profile.ProfileInfo.LastActivityDate"></see>  of a user profile occurs on or before this date and time, the profile is considered inactive.</param>
        /// <returns>
        /// The number of profiles in which the last activity date occurred on or before the specified date.
        /// </returns>
        public override int GetNumberOfInactiveProfiles(System.Web.Profile.ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate)
        {
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);
            ProfileInfoCollection pic = profiles.CreateProfileInfoCollection(authenticationOption);

            int counter = 0;
            foreach (ProfileInfo p in pic)
            {
                if (p.LastActivityDate <= userInactiveSinceDate)
                {
                    counter++;
                }
            }

            return counter;
        }

        /// <summary>
        /// Gets or sets the name of the currently running application.
        /// </summary>
        /// <value></value>
        /// <returns>A <see cref="T:System.String"></see> that contains the application's shortened name, which does not contain a full path or extension, for example, SimpleAppSettings.</returns>
        public override string ApplicationName
        {
            get { return _applicationName; }
            set { _applicationName = value; }
        }

        /// <summary>
        /// Returns the collection of settings property values for the specified application instance and settings property group.
        /// </summary>
        /// <param name="context">A <see cref="T:System.Configuration.SettingsContext"></see> describing the current application use.</param>
        /// <param name="collection">A <see cref="T:System.Configuration.SettingsPropertyCollection"></see> containing the settings property group whose values are to be retrieved.</param>
        /// <returns>
        /// A <see cref="T:System.Configuration.SettingsPropertyValueCollection"></see> containing the values for the specified settings property group.
        /// </returns>
        public override System.Configuration.SettingsPropertyValueCollection GetPropertyValues(System.Configuration.SettingsContext context, System.Configuration.SettingsPropertyCollection collection)
        {
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);
            Collection<Data.Profile> profile = null;

            string userID = context["UserName"].ToString();
            bool authenticated = Convert.ToBoolean(context["IsAuthenticated"]);

            if (authenticated)
            {
                profile = profiles.FindByUserName(userID, ProfileAuthenticationOption.Authenticated);
            }
            else
            {
                profile = profiles.FindByUserName(userID, ProfileAuthenticationOption.Anonymous);
            }


            System.Configuration.SettingsPropertyValueCollection spvc = new System.Configuration.SettingsPropertyValueCollection();
            Data.Profile prof = null;

            if (profile.Count > 0)
            {
                prof = profile[0];
            }


            foreach (System.Configuration.SettingsProperty prop in collection)
            {
                System.Configuration.SettingsPropertyValue val = new System.Configuration.SettingsPropertyValue(prop);
                if (prof != null && prof.Properties.ContainsKey(prop.Name))
                {
                    if (val.Deserialized)
                    {
                        val.PropertyValue = prof.Properties[prop.Name].Value;
                    }
                    else
                    {
                        val.SerializedValue = prof.Properties[prop.Name].Value;
                        if (prop.SerializeAs == System.Configuration.SettingsSerializeAs.ProviderSpecific)
                        {
                            val.PropertyValue = Convert.ChangeType(val.SerializedValue, prop.PropertyType);
                        }
                    }
                    val.IsDirty = false;
                }
                spvc.Add(val);
            }

            return spvc;
        }

        /// <summary>
        /// Sets the values of the specified group of property settings.
        /// </summary>
        /// <param name="context">A <see cref="T:System.Configuration.SettingsContext"></see> describing the current application usage.</param>
        /// <param name="collection">A <see cref="T:System.Configuration.SettingsPropertyValueCollection"></see> representing the group of property settings to set.</param>
        public override void SetPropertyValues(System.Configuration.SettingsContext context, System.Configuration.SettingsPropertyValueCollection collection)
        {
            Data.ProfileCollection profiles = Data.ProfileCollection.Load(this.DataStoreFile);
            Collection<Data.Profile> profile = null;

            string userID = context["UserName"].ToString();
            bool authenticated = Convert.ToBoolean(context["IsAuthenticated"]);

            if (authenticated)
            {
                profile = profiles.FindByUserName(userID, ProfileAuthenticationOption.Authenticated);
            }
            else
            {
                profile = profiles.FindByUserName(userID, ProfileAuthenticationOption.Anonymous);
            }

            Data.Profile p = null;

            if (profile.Count > 0)
            {
                p = profile[0];
            }
            else
            {
                p = new Data.Profile();
                p.IsAnonymous = !authenticated;
                p.UserName = userID;
                profiles.Add(p);
            }

            foreach (System.Configuration.SettingsPropertyValue val in collection)
            {
                if (val.SerializedValue != null)
                {
                    if (p.Properties.ContainsKey(val.Name))
                    {
                        p.Properties[val.Name].Value = val.SerializedValue;
                    }
                    else
                    {
                        Data.ProfileProperty pro = new Data.ProfileProperty(val.Name);
                        pro.Value = val.SerializedValue;
                        p.Properties.Add(pro);
                    }
                }
            }

            profiles.Save(this.DataStoreFile);
        }
    }
}
